## $Id$
##
## Copyright (C) 2022 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import grok
import requests
import urlparse
from waeup.kofa.interfaces import IUniversity
from kofacustom.nigeria.paypal.config import (
        VERIFY_URL_PROD, VERIFY_URL_TEST, VALID_PAYPAL_EMAILS
        )


class IPNListener(grok.View):
    grok.context(IUniversity)
    grok.name('ipn')
    grok.require('waeup.Public')

    def render(self):
        # we need the raw HTTP body to keep order of form vars
        # vars in `form` are mangled by Zope and not in original order anymore
        cache_stream = self.request.bodyStream.cacheStream
        cache_stream.seek(0)
        content = cache_stream.read()
        verified = verify_notification(content)
        return u''


def verify_notification(params_str, verify_url=VERIFY_URL_TEST):
    """Contact paypal to verify a received notification.

    In case of payments happening, paypal will contact us by calling our IPN
    listener with a POST request.

    This request will provide POST vars that have to be returned to Paypal
    slightly modified (and within 30 seconds).

    See https://developer.paypal.com/api/nvp-soap/ipn/IPNIntro/ for details.

    """
    params = [('cmd', '_notify-validate')]
    params.extend(urlparse.parse_qsl(params_str, keep_blank_values=True))
    headers =  {'content-type': 'application/x-www-form-urlencoded',
                'user-agent': 'Kofa IPN listener'}
    req = requests.post(verify_url, params=params, headers=headers, verify=True)
    req.raise_for_status()
    params_dict = dict(params)
    if params_dict.get("receiver_email", None) not in VALID_PAYPAL_EMAILS:
        # we got a notfication for someone else
        return False
    if req.text == 'VERIFIED':  # "VERIFIED", "INVALID" (or something else)
        return True
    return False
